#if 0
#define USE_DTD 1
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmqc.h>
#include <time.h>

#define NUM_MESSAGES 1

#if defined(USE_DTD)
#define XML_MESSAGE \
"<?xml version='1.0' encoding='UTF-8'?>"\
"<!DOCTYPE NETSALES SYSTEM 'NETSALES.dtd'>"\
"<!--NetSales Comment Line-->"\
"<outer>"\
"<mjb1>"\
     "<tag1>data1</tag1>"\
     "<tag2>data2</tag2>"\
     "<tag3>data3</tag3>"\
	 "<inner>"\
	     "<innerMost>"\
             "<tag5>data5</tag5>"\
	     "</innerMost>"\
	 "</inner>"\
     "<tag4>data4</tag4>"\
"</mjb1>"\
"<mjb1>"\
     "<tag1>data1</tag1>"\
     "<tag2>data2</tag2>"\
     "<tag3>data3</tag3>"\
	 "<inner>"\
	     "<innerMost>"\
             "<tag5>data5</tag5>"\
	     "</innerMost>"\
	 "</inner>"\
     "<tag4>data4</tag4>"\
"</mjb1>"\
"</outer>"

#else

#define XML_MESSAGE \
"<outer>"\
     "<tag1>data1</tag1>"\
     "<tag2>data2</tag2>"\
     "<tag3>data3</tag3>"\
	 "<inner>"\
	     "<innerMost>"\
             "<tag5>data5</tag5>"\
	     "</innerMost>"\
	 "</inner>"\
     "<tag4>data4</tag4>"\
"</outer>"

#endif

#define NUM_QUEUES 1
#if defined(WIN32)
    #define QM_DEFAULT "NT_QM"
#elif defined(_AIX)
    #define QM_DEFAULT "SEAIX_QM"
#elif defined(_SOLARIS)
    #define QM_DEFAULT "SUN_QM"
#elif defined(_HPUX)
    #define QM_DEFAULT "HP_QM"
#endif

#define REQUEST_Q "I1.Q"
#define REPLY_Q "O1.Q"

int main(int argc, char *argv[])
{
	MQHCONN hConn;
	MQHOBJ requestHandle, replyHandle;
    MQLONG   openOptions;               /* MQOPEN options                */
    MQOD od = {MQOD_DEFAULT};           /* Object Descriptor */
    MQPMO pmo = {MQPMO_DEFAULT};        /* put message options */
    MQGMO gmo = {MQGMO_DEFAULT};        /* put message options */
	MQMD md,
		 mdDefault = {MQMD_DEFAULT};           /* Message Descriptor */
	MQLONG compCode,
		   reason;
	int msgFlow2 = 0;
	int i,
		j,
		messageLength,
		returnedMessageLength;
        char correlId[NUM_MESSAGES][MQ_CORREL_ID_LENGTH];
	char message[10000],
		 *qmName,
		 *requestQueue,
		 *replyQueue;
	int numMessages = NUM_MESSAGES; 

	/* display usage template if requested */
    if (argc == 2 && argv[1][0] == '?' ||
        argc > 1 && argc != 4)
	{
		printf("Usage: %s < <queueManagerName> <requestQueue> <replyQueue>\n",
			    argv[0]);
		return(0);
	}

    if (argc > 1)
    {
        /* read in queue manager name */
		qmName = argv[1];

		/* now read the queue name(s) */
		requestQueue = argv[2];
		replyQueue = argv[3];
    }
    else
    {
		qmName = QM_DEFAULT;
		requestQueue = REQUEST_Q;
		replyQueue = REPLY_Q;
    }

	printf("Queue manager: %s\n", qmName);
	printf("Request queue: %s\n", requestQueue);
	printf("Reply queue: %s\n", replyQueue);

	/* connect to queue manager, assume default */
	MQCONN(qmName,
		   &hConn,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQCONN failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}

	/* open the request queue */
	strcpy(od.ObjectName, requestQueue);
	/* set the open mode */
	openOptions = MQOO_OUTPUT + MQOO_FAIL_IF_QUIESCING; /* open queue for output */

	MQOPEN(hConn,
		   &od,
		   openOptions,
		   &requestHandle,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQOPEN for input queue '%s' failed with compcode=%d, reason=%d\n",
			   od.ObjectName, compCode, reason);
		exit(1);
	}

	/* open the reply queue */
	strcpy(od.ObjectName, replyQueue);

	/* set the open mode */
	openOptions = MQOO_INPUT_AS_Q_DEF + MQOO_FAIL_IF_QUIESCING; /* open queue for output */

	MQOPEN(hConn,
		   &od,
		   openOptions,
		   &replyHandle,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQOPEN for reply queue '%s' failed with compcode=%d, reason=%d\n",
			   od.ObjectName, compCode, reason);
		exit(1);
	}

	for (i=0; i<numMessages; i++)
	{

		for (j=0; j<NUM_QUEUES; j++)
		{
#if 0
			sprintf(message, "This is iteration %d, request %d",
							 i+1, j+1);
#else
			strcpy(message, XML_MESSAGE);
#endif
			messageLength = strlen(message);
			memcpy(&md, &mdDefault, sizeof(md));
#if 0
			memcpy(md.CorrelId, aggregateId, MQ_CORREL_ID_LENGTH);
#endif
			strcpy(md.ReplyToQ, "O1.Q");

			/* put the message on the queue */
			MQPUT(hConn,                   /* connection handle */
				  requestHandle,               /* object handle */
				  &md,                     /* message descriptor */
				  &pmo,                    /* put options */
				  (MQLONG)messageLength,   /* msg length */
				  message,                 /* message buffer */
				  &compCode,               /* completion code */
				  &reason);                /* reason code */
			if (reason != MQRC_NONE)
			{
				printf("MQPUT failed with compcode=%d, reason=%d\n",
					   compCode, reason);
				exit(1);
			}
		        memcpy(correlId[i], md.MsgId, MQ_MSG_ID_LENGTH);
		}
	}

	/* read the aggregated reply from the reply queue */
	for (i=0; i<numMessages; i++)
	{

		/* read message with matching correl id */
		fprintf(stderr, "about to read aggregate reply %d\n", i+1);
		messageLength = sizeof(message);
		memcpy(&md, &mdDefault, sizeof(md));
		memcpy(md.CorrelId, correlId[i], MQ_CORREL_ID_LENGTH);

		gmo.Options = MQGMO_WAIT +
		              MQGMO_NO_SYNCPOINT;
        gmo.WaitInterval = MQWI_UNLIMITED;

		/* get the message from the queue */
		MQGET(hConn,                   /* connection handle */
			  replyHandle,             /* object handle */
			  &md,                     /* message descriptor */
			  &gmo,                    /* put options */
			  (MQLONG)messageLength,   /* msg length */
			  message,                 /* message buffer */
	  (long *)&returnedMessageLength,  /* returned message length */
			  &compCode,               /* completion code */
			  &reason);                /* reason code */
		if (reason != MQRC_NONE)
		{
			printf("MQGET failed with compcode=%d, reason=%d\n",
				   compCode, reason);
			exit(1);
		}

		fprintf(stderr, "Reply message is:\n'%.*s'\n",
			            returnedMessageLength, message);
	}

	MQDISC(&hConn,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQDISC failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}

    return(0);
}
