#if !defined(__AGGREGATOR_H)
    #define __AGGREGATOR_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : aggregator.h                                             */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for aggregator.c                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Apr 2000                     */
/*                                                                            */
/******************************************************************************/

/* hack to get build to work ok under NT 4.0 */
#if 0  /* no longer seems to be required??? */
	#if !defined(_WIN32_WINNT)
		/* hack to get build to work ok under NT 4.0 */
		#define _WIN32_WINNT 0x0400
	#endif
#endif

#if defined(WIN32)
	#include <windows.h>
	#include <winbase.h>
#endif

#include "unicode.h"  /* must preceede BipCci.h which relies on wchar for NT, AIX */
#include <BipCci.h>	  /* Data types for Plug-in Node functions     */

#include "trace_publ.h"
#include "plugin.h"

#if !defined(TRACE_ALLOC)
	#if defined(TRACE_STORAGE)
		#define TRACE_ALLOC(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): alloc %.8x", __addr));
	#else
		#define TRACE_ALLOC(__key, __addr) 
	#endif
#endif

#if !defined(TRACE_FREE)
	#if defined(TRACE_STORAGE)
		#define TRACE_FREE(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): free %.8x", __addr));
	#else
		#define TRACE_FREE(__key, __addr) 
	#endif
#endif

#if defined(__THREADED)
    #include "threads.h"
#endif

#include <cmqc.h>
#include "utilities.h"  
#include "MQSIV2_aggregator_msg.h"
#include "rc.h"
#include "aggregator_constants.h"

#if defined(_AIX) || defined(_SOLARIS) || defined(_HPUX)
    #include <values.h>
    #define INFINITE MAXINT
#endif

/* constants */
#define MAX_TIMER_NAME_LEN         256
#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif


/* typedefs */
#include <time.h>

/* 
 * NB!!!! - any changes to the following structure should also be reflected in
 * the aggregateMemberLoggedData_t and the aggregateReplyLoggedData_t structures
 * defined in log.h
 */
typedef struct aggregateMember {
	int sequenceNumber,
		messageLength;
	time_t requestArrivalTime,
		   replyArrivalTime;
	char *mqmdP,
		 *messageP;
	CciChar replyToQueue[MQ_Q_NAME_LENGTH+1],
		    replyToQueueManager[MQ_Q_MGR_NAME_LENGTH+1];
	char originalMessageId[MQ_MSG_ID_LENGTH],
	     newMessageId[MQ_MSG_ID_LENGTH];
	struct aggregateInstance *aggregateInstanceP;
	struct msgIdIndexElement *indexP;
	struct aggregateMember *nextP,
		                   *prevP;
} aggregateMember_t;
#define AGGREGATE_MEMBER_DEFAULT 0,    \
	                             0,    \
	                             0,    \
	                             0,    \
								 NULL, \
								 NULL, \
                                 {NULL_48_AND_TRAILING_NULL},   \
                                 {NULL_48_AND_TRAILING_NULL},   \
								 NULL_24,   \
								 NULL_24,   \
								 NULL, \
								 NULL, \
								 NULL, \
								 NULL

typedef struct aggregateInstance {
	char aggregateId[MQ_CORREL_ID_LENGTH],
	     initialMsgId[MQ_MSG_ID_LENGTH];
	int memberCount;
	int outstandingReplies;
	time_t timeoutTime;
	bool_t timedOut,
		   commonMsgId;
	aggregateMember_t memberAnchor;
	struct deployedAggregate *deployedAggregateP;
	struct aggregateInstance *nextP,
		                       *prevP,
	                           *nextAlarmP,
		                       *prevAlarmP;
} aggregateInstance_t;
#define AGGREGATE_INSTANCE_DEFAULT   NULL_24,                   \
	                                 NULL_24,                   \
	                                 0,                         \
									 0,                         \
									 0,                         \
									 FALSE,                     \
									 FALSE,                     \
									{AGGREGATE_MEMBER_DEFAULT}, \
                                     NULL,                      \
                                     NULL,                      \
                                     NULL,                      \
                                     NULL,                      \
									 NULL

typedef enum
{
	REPLY_TYPE_BLOB = 1,
	REPLY_TYPE_XML
} replyType_t;

typedef struct deployedAggregate {
	CciChar *name;
	int timeoutInterval,
		aggregateReplyCount,
		aggregateCreateCount;
	bool_t outputToMQReply,
		   outputToDestinationList,
		   separatorStringSet,
		   persistent,
		   recoveryComplete,
		   aggregateCreateAndReplyExist,
		   setTimeoutCorrelId;
	replyType_t replyType;
	CciChar *replyLabelName;
	CciChar replyToQueueName[MQ_Q_NAME_LENGTH + 1];
	char *separatorString;
	int  separatorStringLength;
	aggregateInstance_t aggregateAnchor;
	struct context *contextP;
	struct messageFlow *messageFlowEntryP;
	struct deployedAggregate *nextP,
		                     *prevP;
} deployedAggregate_t;
#define DEPLOYED_AGGREGATE_DEFAULT NULL,             /* name */                        \
	                               0,                /* timeoutInterval */             \
	                               0,                /* aggregateReplyCount */         \
	                               0,                /* aggregateCreateCount */        \
                                   FALSE,            /* outputToMQReply */             \
                                   FALSE,            /* outputToDestinationList */     \
                                   FALSE,            /* separatorStringSet */          \
                                   FALSE,            /* persistent */                  \
                                   FALSE,            /* recoveryComplete */            \
								   FALSE,            /* aggregateCreateAndReplyExist */\
								   FALSE,            /* setTimeoutCorrelId */          \
								   REPLY_TYPE_BLOB,  /* replyType */                   \
								   NULL,             /* replyLabelName */              \
				        NULL_48_AND_TRAILING_NULL,   /* replyToQueueName */            \
								   NULL,             /* separatorString */             \
                                   0,                /* separatorStringLength */       \
                                   {AGGREGATE_INSTANCE_DEFAULT},/* aggregateAnchor */  \
                                   NULL,             /* contextP */                    \
                                   NULL,             /* messageFlowEntryP */           \
                                   NULL,             /* nextP */                       \
								   NULL              /* prevP */

typedef enum
{
	AGGREGATE_CREATE = 1,
	AGGREGATE_REPLY
} aggregateType_t;

#include "log.h"

extern char *traceRetcodeStrings[];
extern int traceMaxRetcodeValue;

typedef struct addressList {
	int numAddresses,
		nextAddress;
	void *address[1];
} addressList_t;


#if defined(GLOBAL_AGGREGATOR_DATA)
	/* global variables */
	addressList_t *addrListP = NULL;
    threadMutex_t aggregateChainMutex,
				  alarmThreadSyncMutex,
				  logThreadSyncMutex,
	  			  logThreadWorkMutex,
	  			  uuidGenMutex;
	threadCondVar_t alarmThreadWorkCondVar,
		            alarmThreadSyncCondVar,
		            logThreadWorkCondVar,
		            logThreadSyncCondVar;
	bool_t shutdownRequested = FALSE,
		   alarmThreadInitialised = FALSE,
		   alarmThreadCreated = FALSE,
           logThreadInitialised = FALSE,
           logThreadCreated = FALSE,
		   brokerInfoObtained = FALSE,
		   multipleExecutionGroups = FALSE,
		   alarmThreadShuttingDown = FALSE,
		   logThreadShuttingDown = FALSE,
		   firstTraceCall = TRUE;

	messageFlow_t messageFlowAnchor = {MESSAGE_FLOW_DEFAULT};
    deployedAggregate_t deployedAggregateAnchor = {DEPLOYED_AGGREGATE_DEFAULT},
		                *deployedAggregateAnchorP = &deployedAggregateAnchor;
	aggregateInstance_t alarmAnchor = {AGGREGATE_INSTANCE_DEFAULT},
		                  timedOutAnchor = {AGGREGATE_INSTANCE_DEFAULT},
						  *timedOutAnchorP = &timedOutAnchor;
	msgIdIndexElement_t msgIdIndexAnchor = {MESSAGE_ID_INDEX_DEFAULT};
    logTruncationRequest_t logTruncationRequestAnchor = {LOG_TRUNCATION_REQUEST_DEFAULT},
		                   *logTruncationRequestAnchorP = &logTruncationRequestAnchor;
	int numLogTruncationRequests = 0;
	char *alarmThreadErrorMessage = NULL,
	     *logThreadErrorMessage = NULL;
	threadHandle_t alarmThreadHandle,
		           logThreadHandle;
	
	char queueManagerName[MQ_Q_MGR_NAME_LENGTH+1] = "";
	char brokerName[MAX_STRING_SIZE+1] = "";
	char executionGroupName[MAX_STRING_SIZE+1] = "";

	CciChar queueManagerNameUC[MQ_Q_MGR_NAME_LENGTH+1];
	CciChar brokerNameUC[MAX_STRING_SIZE+1];
	CciChar executionGroupNameUC[MAX_STRING_SIZE+1];

	/* trace stuff */
	char *callersTraceEnvironmentVariable = "AGGREGATOR_PLUGIN_TRACE";
	int traceCallerIndex;
	threadKey_t loggingKey = 0;

#else
extern addressList_t *addrListP;
	/* global variables */
    extern threadMutex_t aggregateChainMutex,
				         alarmThreadSyncMutex,
				         logThreadSyncMutex,
						 logThreadWorkMutex,
	  			         uuidGenMutex;
	extern threadCondVar_t alarmThreadWorkCondVar,
		                   alarmThreadSyncCondVar,
		                   logThreadWorkCondVar,
		                   logThreadSyncCondVar;
	extern bool_t shutdownRequested,
		          alarmThreadInitialised,
		          alarmThreadCreated,
                  logThreadInitialised,
                  logThreadCreated,
		          brokerInfoObtained,
		          multipleExecutionGroups,
				  alarmThreadShuttingDown,
				  logThreadShuttingDown,
		          firstTraceCall;
	extern messageFlow_t messageFlowAnchor;
    extern deployedAggregate_t deployedAggregateAnchor,
		                      *deployedAggregateAnchorP;
	extern aggregateInstance_t alarmAnchor,
	                             timedOutAnchor,
						         *timedOutAnchorP;
	extern msgIdIndexElement_t msgIdIndexAnchor;
    extern logTruncationRequest_t logTruncationRequestAnchor,
		                          *logTruncationRequestAnchorP;
	extern int numLogTruncationRequests;
	extern char *alarmThreadErrorMessage,
	            *logThreadErrorMessage;
	extern threadHandle_t alarmThreadHandle,
		                  logThreadHandle;

	extern char queueManagerName[];
	extern char brokerName[];
	extern char executionGroupName[];

	extern CciChar queueManagerNameUC[];
	extern CciChar brokerNameUC[];
	extern CciChar executionGroupNameUC[];
	
	/* trace stuff */
	extern char *callersTraceEnvironmentVariable;
	extern int traceCallerIndex;
	extern threadKey_t loggingKey;
#endif

#define INSERT_AFTER_IN_ALARM_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextAlarmP = __currentElementP->nextAlarmP; \
__newElementP->prevAlarmP = __currentElementP; \
if (__currentElementP->nextAlarmP) __currentElementP->nextAlarmP->prevAlarmP = __newElementP; \
__currentElementP->nextAlarmP = __newElementP; 

/* remove from double-linked list */
#define REMOVE_FROM_LINKED_LIST(__currentElementP) \
__currentElementP->prevP->nextP = __currentElementP->nextP; \
if (__currentElementP->nextP) \
	__currentElementP->nextP->prevP = __currentElementP->prevP;

#define REMOVE_FROM_ALARM_CHAIN(__currentElementP) \
__currentElementP->prevAlarmP->nextAlarmP = __currentElementP->nextAlarmP; \
if (__currentElementP->nextAlarmP) \
	__currentElementP->nextAlarmP->prevAlarmP = __currentElementP->prevAlarmP;

#define RETHROW_IF_EXCEPTION(__rc) \
if (__rc == TT_PROPAGATION_EXCEPTION) \
		{ \
	/* about to re-throw last exception, reset the trace indentation first */ \
	TRACE_MID(TRACE_INFO, ("About to rethrow exception, resetting trace indentation")); \
	TraceSetIndentationValue(0); \
	cciRethrowLastException(0); \
}

/* Function Prototypes */
int evaluateAggregateCreate(context_t *contextP,
		                   CciMessage *destinationList,
		                   CciMessage *exceptionList,
		                   CciMessage *message);

int evaluateAggregateReply(context_t *contextP,
		                   CciMessage *destinationList,
		                   CciMessage *exceptionList,
		                   CciMessage *message);

void setAggregateReplyAlarm(IN aggregateInstance_t *currentAggregateP,
	  	                    IN aggregateMember_t *memberP,
							IN bool_t recoveryProcessing);

int processAttribute(IN context_t *contextP,
				     IN CciChar *attributeName,
				     IN CciChar *attributeValue);

deployedAggregate_t *findDeployedAggregate(IN CciChar *name);

aggregateInstance_t *findAggregateInstanceFromContext(IN context_t *contextP,
														IN char *aggregateId);

void freeAllAggregateMembers(IN aggregateInstance_t *aggregateP);

threadFunction_t processAlarms(void *dataP);

threadFunction_t truncateLog(void *dataP);

void updateAlarmPairsTable(void *contextP,
						   CciChar *alarmName);

int notifyAggregateReplyNodeOfExpiredAggregate(IN MQHCONN qmConn,
											   IN aggregateInstance_t *aggregateP);

int buildCompletedAggregateReplyMessage(IN CciMessage *message,
						                IN aggregateInstance_t *aggregateP,
							            OUT CciMessage **newMessageP);

int processExpiredAggregate(IN context_t *contextP,
						    IN aggregateInstance_t *aggregateP,
                            IN CciMessage *message,
		   		            IN CciMessage *destinationList,
	                        IN CciMessage *exceptionList);

int buildAggregateMessagePayload(IN aggregateInstance_t *aggregateP,
								 OUT char **bufferP,
								 OUT int *lengthP,
								 OUT int *numberOfRepliesP);

int addAddressToDestinationList(IN CciMessage *destinationList,
								IN CciChar *queueName,
								IN CciChar *queueManagerName,
								OUT CciMessage **newDestinationListP);

void removeMsgIdIndex(IN msgIdIndexElement_t *msgIndexP);

int createAlarmThread(IN context_t *contextP);

void *getMessageIdElement(IN void *message);

void addMemberToAggregateInstance(IN aggregateMember_t *memberP,
		                          IN aggregateInstance_t *aggregateP);

void *replaceMessageCorrelId(IN void *message, 
							 IN char *correlId);

int cleanUpAggregateAfterException(IN context_t* contextP,
						           IN char *aggregateId);

void removeAllAggregateMsgIndexEntries(aggregateInstance_t *aggregateP);

#if 0
void deleteAggregateInstances(void);
#else
void deleteAggregateInstances(IN context_t *contextP,
							  IN deployedAggregate_t *deployedAggregateP);
#endif

int processTimerMessage(IN context_t *contextP,
 			   	        IN CciMessage *destinationList,
		                IN CciMessage *exceptionList,
		                IN CciMessage *message);

int deleteMessageBody(INOUT CciMessage *message);

int replaceMessageBodyWithBLOB(INOUT CciMessage *message,
						       IN char *payload,
							   IN int payloadLength);

int writeAggregateInstanceToDB(IN context_t* contextP,
							   IN aggregateInstance_t *aggregateInstanceP,
 			   		           IN CciMessage *destinationList,
		                       IN CciMessage *exceptionList,
		                       IN CciMessage *message);

int writeMsgIdToDB(IN context_t* contextP,
				   IN char *msgId,
 		 	   	   IN CciMessage *destinationList,
		           IN CciMessage *exceptionList,
		           IN CciMessage *message);

int readMsgIdFromDB(IN context_t* contextP,
				    OUT char *msgId,
 		 	   	    IN CciMessage *destinationList,
		            IN CciMessage *exceptionList,
		            IN CciMessage *message);

void readMsgData(IN CciMessage *msg);

void traverseMessage(IN void* element, 
					 IN int blankCount,
					 IN int childLevel);

/* Node Implementation Functions */
void *createNodeContext(void    *factoryObject,
                        CciChar *nodeName,
                        void    *nodeObject);

int setAttribute(void    *context,
                 CciChar *attributeName,
                 CciChar *attributeValue);

void evaluate(void *context, 
              void *destinationList,
              void *exceptionList,
              void *message);

int genMessageId(OUT char *msgId);

void runTheChain();
void showGlobals(IN deployedAggregate_t *deployedAggregateP);

#if 0
void testAddress(int label);
#endif

#endif

