#if !defined(__AGGREGATOR_CONSTANTS_H)
    #define __AGGREGATOR_CONSTANTS_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : aggregator_constants.h                                   */
/*                                                                            */
/* Description     : This header file contains unicode constants for the      */
/*                   aggregator plug-in node.                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Jul 2001                     */
/*                                                                            */
/******************************************************************************/

#include <BipCci.h>	  /* Data types for Plug-in Node functions     */

#if defined(WIN32)  // little endian, so no need to shift bytes 
	#define UC(__char) ((CciChar)__char)
	#define UCB ((CciChar)' ')
	#define UCDOT ((CciChar)'.')
#elif defined(_SOLARIS) || defined(_HPUX)
	#define UC(__char) (((CciChar)__char))
	#define UCB (((CciChar)' '))
	#define UCDOT (((CciChar)'.'))
#else
	#define UC(__char) (((CciChar)__char)<<8)
	#define UCB (((CciChar)' ')<<8)
	#define UCDOT (((CciChar)'.')<<8)
#endif

typedef struct aggregatorConstants {
    CciChar 
        *PLUGIN_MESSAGE_CATALOGUE,
        *AGGREGATE_CREATE_NODE,
        *AGGREGATE_REPLY_NODE,
        *ATTR_AGGREGATE_NAME,
        *ATTR_SEQUENCE_NUMBER,
        *ATTR_SEQUENCE_NUMBER_DEFAULT_VALUE,
        *ATTR_REPLY_TIMEOUT_SECONDS,
        *ATTR_REPLY_TIMEOUT_SECONDS_DEFAULT_VALUE,
        *ATTR_REPLY_TO_QUEUE,
        *ATTR_SEPARATOR_STRING,
        *ATTR_HEX_SEPARATOR_STRING,
        *ATTR_AGGREGATE_REPLY_DESTINATION,
        *ATTR_AGGREGATE_REPLY_DESTINATION_DEFAULT_VALUE,
        *ATTR_REPLY_TYPE,
        *ATTR_REPLY_TYPE_DEFAULT_VALUE,
        *ATTR_REPLY_LABEL_NAME,
        *ATTR_PERSISTENT,
        *ATTR_PERSISTENT_DEFAULT_VALUE,
        *ATTR_LABEL,
        *ATTR_USER_TRACE_LEVEL,
        *ATTR_TRACE_LEVEL,
        *ATTR_USER_TRACE_FILTER,
        *ATTR_TRACE_FILTER,
        *TERM_IN,
        *TERM_OUT,
        *TERM_FAILURE,
        *TERM_TIMED_OUT,
        *TERM_LATE_REPLIES,
        *AGGREGATOR_PLUGIN_FACTORY,
        *REPLY_TO_QUEUE,
        *BLOB,
        *ROOT_XML,
        *ROOT_XML_DOT,
        *DOT,
        *NULL_STRING,
        *NONE,
        *DESTINATION,
        *DESTINATION_LIST,
        *MQ_DESTINATION_LIST,
        *DESTINATION_DATA,
        *FORMAT_NONE,
        *FORMAT_STRING,
        *QUEUE_MANAGER_NAME,
        *QUEUE_NAME,
        *XML,
        *TIMER_THREAD_ID,
        BLANK_CHAR;
} aggregatorConstants_t;

/* build static initialisation values for Solaris */
#if defined(_SOLARIS) || defined(_HPUX)

#if defined(GLOBAL_AGGREGATOR_DATA)
CciChar 
    PLUGIN_MESSAGE_CATALOGUE_VAL[sizeof("MQSIV2_aggregator")*sizeof(CciChar)] =
    {'M','Q','S','I','V','2','_','a','g','g','r','e','g','a','t','o','r',0},
    AGGREGATE_CREATE_NODE_VAL[sizeof("MJBAggregateCreateNode")*sizeof(CciChar)] =
    {'M','J','B','A','g','g','r','e','g','a','t','e','C','r','e','a','t','e','N','o','d','e',0},
    AGGREGATE_REPLY_NODE_VAL[sizeof("MJBARINode")*sizeof(CciChar)] =
    {'M','J','B','A','R','I','N','o','d','e',0},
    ATTR_AGGREGATE_NAME_VAL[sizeof("AggregateName")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','e','N','a','m','e',0},
    ATTR_SEQUENCE_NUMBER_VAL[sizeof("SequenceNumber")*sizeof(CciChar)] =
    {'S','e','q','u','e','n','c','e','N','u','m','b','e','r',0},
    ATTR_SEQUENCE_NUMBER_DEFAULT_VALUE_VAL[sizeof("0")*sizeof(CciChar)] =
    {'0',0},
    ATTR_REPLY_TIMEOUT_SECONDS_VAL[sizeof("ReplyTimeoutSeconds")*sizeof(CciChar)] =
    {'R','e','p','l','y','T','i','m','e','o','u','t','S','e','c','o','n','d','s',0},
    ATTR_REPLY_TIMEOUT_SECONDS_DEFAULT_VALUE_VAL[sizeof("5")*sizeof(CciChar)] =
    {'5',0},
    ATTR_REPLY_TO_QUEUE_VAL[sizeof("ReplyToQueue")*sizeof(CciChar)] =
    {'R','e','p','l','y','T','o','Q','u','e','u','e',0},
    ATTR_SEPARATOR_STRING_VAL[sizeof("SeparatorString")*sizeof(CciChar)] =
    {'S','e','p','a','r','a','t','o','r','S','t','r','i','n','g',0},
    ATTR_HEX_SEPARATOR_STRING_VAL[sizeof("HexSeparatorString")*sizeof(CciChar)] =
    {'H','e','x','S','e','p','a','r','a','t','o','r','S','t','r','i','n','g',0},
    ATTR_AGGREGATE_REPLY_DESTINATION_VAL[sizeof("AggregatedReplyDestination")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','e','d','R','e','p','l','y','D','e','s','t','i','n','a','t','i','o','n',0},
    ATTR_AGGREGATE_REPLY_DESTINATION_DEFAULT_VALUE_VAL[sizeof("default")*sizeof(CciChar)] =
    {'d','e','f','a','u','l','t',0},
    ATTR_REPLY_TYPE_VAL[sizeof("AggregatedReplyDataType")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','e','d','R','e','p','l','y','D','a','t','a','T','y','p','e',0},
    ATTR_REPLY_TYPE_DEFAULT_VALUE_VAL[sizeof("BLOB")*sizeof(CciChar)] =
    {'B','L','O','B',0},
    ATTR_REPLY_LABEL_NAME_VAL[sizeof("AggregatedReplyLabelName")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','e','d','R','e','p','l','y','L','a','b','e','l','N','a','m','e',0},
    ATTR_PERSISTENT_VAL[sizeof("Recoverable")*sizeof(CciChar)] =
    {'R','e','c','o','v','e','r','a','b','l','e',0},
    ATTR_PERSISTENT_DEFAULT_VALUE_VAL[sizeof("false")*sizeof(CciChar)] =
    {'f','a','l','s','e',0},
    ATTR_LABEL_VAL[sizeof("label")*sizeof(CciChar)] =
    {'l','a','b','e','l',0},
    ATTR_USER_TRACE_LEVEL_VAL[sizeof("userTraceLevel")*sizeof(CciChar)] =
    {'u','s','e','r','T','r','a','c','e','L','e','v','e','l',0},
    ATTR_TRACE_LEVEL_VAL[sizeof("traceLevel")*sizeof(CciChar)] =
    {'t','r','a','c','e','L','e','v','e','l',0},
    ATTR_USER_TRACE_FILTER_VAL[sizeof("userTraceFilter")*sizeof(CciChar)] =
    {'u','s','e','r','T','r','a','c','e','F','i','l','t','e','r',0},
    ATTR_TRACE_FILTER_VAL[sizeof("traceFilter")*sizeof(CciChar)] =
    {'t','r','a','c','e','F','i','l','t','e','r',0},
    TERM_IN_VAL[sizeof("in")*sizeof(CciChar)] =
    {'i','n',0},
    TERM_OUT_VAL[sizeof("out")*sizeof(CciChar)] =
    {'o','u','t',0},
    TERM_FAILURE_VAL[sizeof("failure")*sizeof(CciChar)] =
    {'f','a','i','l','u','r','e',0},
    TERM_TIMED_OUT_VAL[sizeof("timedOut")*sizeof(CciChar)] =
    {'t','i','m','e','d','O','u','t',0},
    TERM_LATE_REPLIES_VAL[sizeof("lateReplies")*sizeof(CciChar)] =
    {'l','a','t','e','R','e','p','l','i','e','s',0},
    AGGREGATOR_PLUGIN_FACTORY_VAL[sizeof("AggregatorPlugInFactory")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','o','r','P','l','u','g','I','n','F','a','c','t','o','r','y',0},
    REPLY_TO_QUEUE_VAL[sizeof("ReplyToQueue")*sizeof(CciChar)] =
    {'R','e','p','l','y','T','o','Q','u','e','u','e',0},
    BLOB_VAL[sizeof("BLOB")*sizeof(CciChar)] =
    {'B','L','O','B',0},
    ROOT_XML_VAL[sizeof("Root.XML")*sizeof(CciChar)] =
    {'R','o','o','t','.','X','M','L',0},
    ROOT_XML_DOT_VAL[sizeof("Root.XML.")*sizeof(CciChar)] =
    {'R','o','o','t','.','X','M','L','.',0},
    DOT_VAL[sizeof(".")*sizeof(CciChar)] =
    {'.',0},
    NULL_STRING_VAL[sizeof("")*sizeof(CciChar)] =
    {0},
    NONE_VAL[sizeof("none")*sizeof(CciChar)] =
    {'n','o','n','e',0},
    DESTINATION_VAL[sizeof("Destination")*sizeof(CciChar)] =
    {'D','e','s','t','i','n','a','t','i','o','n',0},
    DESTINATION_LIST_VAL[sizeof("DestinationList")*sizeof(CciChar)] =
    {'D','e','s','t','i','n','a','t','i','o','n','L','i','s','t',0},
    MQ_DESTINATION_LIST_VAL[sizeof("MQDestinationList")*sizeof(CciChar)] =
    {'M','Q','D','e','s','t','i','n','a','t','i','o','n','L','i','s','t',0},
    DESTINATION_DATA_VAL[sizeof("DestinationData")*sizeof(CciChar)] =
    {'D','e','s','t','i','n','a','t','i','o','n','D','a','t','a',0},
    FORMAT_NONE_VAL[sizeof("        ")*sizeof(CciChar)] =
    {' ',' ',' ',' ',' ',' ',' ',' ',0},
    FORMAT_STRING_VAL[sizeof("MQSTR   ")*sizeof(CciChar)] =
    {'M','Q','S','T','R',' ',' ',' ',0},
    QUEUE_MANAGER_NAME_VAL[sizeof("queueManagerName")*sizeof(CciChar)] =
    {'q','u','e','u','e','M','a','n','a','g','e','r','N','a','m','e',0},
    QUEUE_NAME_VAL[sizeof("queueName")*sizeof(CciChar)] =
    {'q','u','e','u','e','N','a','m','e',0},
    XML_VAL[sizeof("XML")*sizeof(CciChar)] =
    {'X','M','L',0},
    TIMER_THREAD_ID_VAL[sizeof("AggregatorTimerThread       ")*sizeof(CciChar)] =
    {'A','g','g','r','e','g','a','t','o','r','T','i','m','e','r','T','h','r','e','a','d',' ',' ',' ',' ',' ',' ',' ',0};
#else
extern CciChar 
    PLUGIN_MESSAGE_CATALOGUE_VAL[],
    AGGREGATE_CREATE_NODE_VAL[],
    AGGREGATE_REPLY_NODE_VAL[],
    ATTR_AGGREGATE_NAME_VAL[],
    ATTR_SEQUENCE_NUMBER_VAL[],
    ATTR_SEQUENCE_NUMBER_DEFAULT_VALUE_VAL[],
    ATTR_REPLY_TIMEOUT_SECONDS_VAL[],
    ATTR_REPLY_TIMEOUT_SECONDS_DEFAULT_VALUE_VAL[],
    ATTR_REPLY_TO_QUEUE_VAL[],
    ATTR_SEPARATOR_STRING_VAL[],
    ATTR_HEX_SEPARATOR_STRING_VAL[],
    ATTR_AGGREGATE_REPLY_DESTINATION_VAL[],
    ATTR_AGGREGATE_REPLY_DESTINATION_DEFAULT_VALUE_VAL[],
    ATTR_REPLY_TYPE_VAL[],
    ATTR_REPLY_TYPE_DEFAULT_VALUE_VAL[],
    ATTR_REPLY_LABEL_NAME_VAL[],
    ATTR_PERSISTENT_VAL[],
    ATTR_PERSISTENT_DEFAULT_VALUE_VAL[],
    ATTR_LABEL_VAL[],
    ATTR_USER_TRACE_LEVEL_VAL[],
    ATTR_TRACE_LEVEL_VAL[],
    ATTR_USER_TRACE_FILTER_VAL[],
    ATTR_TRACE_FILTER_VAL[],
    TERM_IN_VAL[],
    TERM_OUT_VAL[],
    TERM_FAILURE_VAL[],
    TERM_TIMED_OUT_VAL[],
    TERM_LATE_REPLIES_VAL[],
    AGGREGATOR_PLUGIN_FACTORY_VAL[],
    REPLY_TO_QUEUE_VAL[],
    BLOB_VAL[],
    ROOT_XML_VAL[],
    ROOT_XML_DOT_VAL[],
    DOT_VAL[],
    NULL_STRING_VAL[],
    NONE_VAL[],
    DESTINATION_VAL[],
    DESTINATION_LIST_VAL[],
    MQ_DESTINATION_LIST_VAL[],
    DESTINATION_DATA_VAL[],
    FORMAT_NONE_VAL[],
    FORMAT_STRING_VAL[],
    QUEUE_MANAGER_NAME_VAL[],
    QUEUE_NAME_VAL[],
    XML_VAL[],
    TIMER_THREAD_ID_VAL[];
#endif
#endif


/* define the initialisation macro */
#if defined(WIN32) || defined(_AIX)
#define AGGREGATOR_CONSTANTS_DEFAULTS \
    L"MQSIV2_aggregator", \
    L"MJBAggregateCreateNode", \
    L"MJBARINode", \
    L"AggregateName", \
    L"SequenceNumber", \
    L"0", \
    L"ReplyTimeoutSeconds", \
    L"5", \
    L"ReplyToQueue", \
    L"SeparatorString", \
    L"HexSeparatorString", \
    L"AggregatedReplyDestination", \
    L"default", \
    L"AggregatedReplyDataType", \
    L"BLOB", \
    L"AggregatedReplyLabelName", \
    L"Recoverable", \
    L"false", \
    L"label", \
    L"userTraceLevel", \
    L"traceLevel", \
    L"userTraceFilter", \
    L"traceFilter", \
    L"in", \
    L"out", \
    L"failure", \
    L"timedOut", \
    L"lateReplies", \
    L"AggregatorPlugInFactory", \
    L"ReplyToQueue", \
    L"BLOB", \
    L"Root.XML", \
    L"Root.XML.", \
    L".", \
    L"", \
    L"none", \
    L"Destination", \
    L"DestinationList", \
    L"MQDestinationList", \
    L"DestinationData", \
    L"        ", \
    L"MQSTR   ", \
    L"queueManagerName", \
    L"queueName", \
    L"XML", \
    L"AggregatorTimerThread       ", \
    L' '


#elif defined(_SOLARIS) || defined(_HPUX)

#define AGGREGATOR_CONSTANTS_DEFAULTS \
    PLUGIN_MESSAGE_CATALOGUE_VAL, \
    AGGREGATE_CREATE_NODE_VAL, \
    AGGREGATE_REPLY_NODE_VAL, \
    ATTR_AGGREGATE_NAME_VAL, \
    ATTR_SEQUENCE_NUMBER_VAL, \
    ATTR_SEQUENCE_NUMBER_DEFAULT_VALUE_VAL, \
    ATTR_REPLY_TIMEOUT_SECONDS_VAL, \
    ATTR_REPLY_TIMEOUT_SECONDS_DEFAULT_VALUE_VAL, \
    ATTR_REPLY_TO_QUEUE_VAL, \
    ATTR_SEPARATOR_STRING_VAL, \
    ATTR_HEX_SEPARATOR_STRING_VAL, \
    ATTR_AGGREGATE_REPLY_DESTINATION_VAL, \
    ATTR_AGGREGATE_REPLY_DESTINATION_DEFAULT_VALUE_VAL, \
    ATTR_REPLY_TYPE_VAL, \
    ATTR_REPLY_TYPE_DEFAULT_VALUE_VAL, \
    ATTR_REPLY_LABEL_NAME_VAL, \
    ATTR_PERSISTENT_VAL, \
    ATTR_PERSISTENT_DEFAULT_VALUE_VAL, \
    ATTR_LABEL_VAL, \
    ATTR_USER_TRACE_LEVEL_VAL, \
    ATTR_TRACE_LEVEL_VAL, \
    ATTR_USER_TRACE_FILTER_VAL, \
    ATTR_TRACE_FILTER_VAL, \
    TERM_IN_VAL, \
    TERM_OUT_VAL, \
    TERM_FAILURE_VAL, \
    TERM_TIMED_OUT_VAL, \
    TERM_LATE_REPLIES_VAL, \
    AGGREGATOR_PLUGIN_FACTORY_VAL, \
    REPLY_TO_QUEUE_VAL, \
    BLOB_VAL, \
    ROOT_XML_VAL, \
    ROOT_XML_DOT_VAL, \
    DOT_VAL, \
    NULL_STRING_VAL, \
    NONE_VAL, \
    DESTINATION_VAL, \
    DESTINATION_LIST_VAL, \
    MQ_DESTINATION_LIST_VAL, \
    DESTINATION_DATA_VAL, \
    FORMAT_NONE_VAL, \
    FORMAT_STRING_VAL, \
    QUEUE_MANAGER_NAME_VAL, \
    QUEUE_NAME_VAL, \
    XML_VAL, \
    TIMER_THREAD_ID_VAL, \
    ' '
#endif


/* finally, define the global initializer structure */
#if defined(GLOBAL_AGGREGATOR_DATA)
    aggregatorConstants_t C = {AGGREGATOR_CONSTANTS_DEFAULTS};
#else
    extern aggregatorConstants_t C;
#endif


#define AGGREGATE_CREATE_NODE_EXTERNAL_NAME_MB "SpAggregateCreate"
#define AGGREGATE_REPLY_NODE_EXTERNAL_NAME_MB "SpAggregateReply"
#define TIMER_THREAD_ID_MB "AggregatorTimerThread       "
#define AGG_TIMER_ID_MB "Agg-"


#define NULL_24 \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
#define NULL_48 \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
#define NULL_48_AND_TRAILING_NULL \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

#endif
