#if !defined(__LOG_H)
#define __LOG_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Progarm name    : log.h                                                    */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   structures used for logging aggregator state data        */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: May 2001                     */
/*                                                                            */
/******************************************************************************/
#include "aggregator.h"

#define AGGREGATOR_RECOVERY_QUEUE "SYSTEM.BROKER.AGGREGATOR.RECOVERY.QUEUE"
#define DEFAULT_LOG_RECORD_SIZE 10000
#define LOG_TRUNCATION_INTERVAL (6000 * 1000)  /* browse log every 60 minutes */

typedef enum
{
	NEW_INSTANCE = 1,
	MEMBER,
    REPLY,
	FINISH
} logRecordType_t;

typedef struct aggregateMemberLoggedData {
	int sequenceNumber,
		messageLength;
	time_t requestArrivalTime,
		   replyArrivalTime;
	char *mqmdP,
		 *messageP;
	CciChar replyToQueue[MQ_Q_NAME_LENGTH+1],
		    replyToQueueManager[MQ_Q_MGR_NAME_LENGTH+1];
	char originalMessageId[MQ_MSG_ID_LENGTH],
	     newMessageId[MQ_MSG_ID_LENGTH];
} aggregateMemberLoggedData_t;

typedef struct aggregateReplyLoggedData {
	int sequenceNumber,
		messageLength;
	time_t requestArrivalTime,
		   replyArrivalTime;
} aggregateReplyLoggedData_t;

typedef struct logRecordHeader {
    logRecordType_t recordType;
	char aggregateId[MQ_CORREL_ID_LENGTH];
} logRecordHeader_t;

typedef struct aggregateNameRecord {
	int nameLength;
    CciChar name[MAX_STRING_SIZE + 1];
} aggregateNameRecord_t;

typedef struct newInstanceRecord {
	int nameLength;
    CciChar messageFlowName[1];
} newInstanceRecord_t;

typedef struct logMember {
	time_t instanceTimeoutTime;
	aggregateMemberLoggedData_t loggedData;
} logMember_t;

typedef struct logReply {
	aggregateReplyLoggedData_t loggedData;
	MQMD md;
} logReply_t;

typedef struct queueDetails {
	MQHCONN qmConn;
	MQHOBJ queueHandle;
} queueDetails_t;

typedef struct logTruncationRequest {
	char aggregateId[MQ_CORREL_ID_LENGTH];
	struct logTruncationRequest *nextP,
		                        *prevP;
} logTruncationRequest_t;

#define LOG_TRUNCATION_REQUEST_DEFAULT "",   \
	                                   NULL, \
	                                   NULL

/*
 *  Global variables.
 */

#if defined(DECLARE_LOG_DATA)
#endif

/* function prototypes */
int logNewInstance(IN aggregateInstance_t *aggregateInstanceP,
				   OUT char **errorStringP);

int logMember(IN aggregateMember_t *memberP,
			  OUT char **errorStringP);

int logReply(IN aggregateMember_t *memberP,
			 OUT char **errorStringP);

int logFinish(IN char *aggregateId,
			  IN CciChar *aggregateName,
			  OUT char **errorStringP);

int logRemoveEntries(IN char *aggregateId,
					 IN bool_t commit,
				     OUT char **errorStringP);

int logRecoverInstances(IN context_t *contextP,
						IN deployedAggregate_t *deployedAggregateP,
						OUT bool_t *instancesRecoveredP,
						OUT char **errorStringP);

#endif
