#if !defined(__THREADS_H)
#define __THREADS_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Progarm name    : threads.h                                                */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for threads.c                                  */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Feb 2000                     */
/*                                                                            */
/******************************************************************************/

#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif


#if defined(WIN32)
  /* hack to get build to work ok under NT 4.0 */
  #define _WIN32_WINNT 0x0400

  #include <windows.h>
  #include <winbase.h>
#elif defined(_AIX)
  #include <pthread.h>
#elif defined(_SOLARIS)
  #include <thread.h>
  #include <synch.h>
#elif defined(_HPUX)
  #include <pthread.h>
#endif /* WIN32 */

#include <stdio.h>
#include <stdarg.h>

#define THREAD_OK 0
#define THREAD_FAILED 1
#define THREAD_TIMED_OUT 2

#if defined(__THREADED)

#if defined(WIN32)

/************************************************/
/************************************************/
/*                                              */
/*      Windows NT threads implementation       */
/*                                              */
/************************************************/
/************************************************/

/*
 * typedefs 
 */
typedef DWORD threadKey_t;
typedef HANDLE threadMutex_t;
typedef HANDLE processLock_t;
typedef HANDLE processLockStorage_t;

typedef struct threadCondVar
{
	threadMutex_t mutex;
	HANDLE      semaphore;
	int         waiterCount;
} *threadCondVar_t, **threadCondVarP_t;

typedef struct threadHandle
{
	HANDLE handle;
    DWORD threadId;
} *threadHandle_t, **threadHandleP_t;

typedef struct processHandle
{
	HANDLE handle;
    DWORD processId;
} *processHandle_t, **processHandleP_t;

typedef LPTHREAD_START_ROUTINE threadFunction_t;

typedef void (*threadOnceFunction_t)(void);

typedef DWORD *FUNCTION_RETURN_TYPE;
typedef void ONCE_FUNCTION_RETURN_TYPE;

typedef void *threadParameter_t;

typedef struct threadOnceBlock
{
	bool_t initialised;
	CRITICAL_SECTION cs;
} threadOnceBlock_t;

#define THREAD_ONCE_BLOCK_INITIALISE {FALSE}

/* if defined(WIN32) */

#elif defined(_AIX)

/************************************************/
/************************************************/
/*                                              */
/*          AIX threads implementation          */
/*                                              */
/************************************************/
/************************************************/

/*
 * typedefs 
 */
typedef pthread_key_t threadKey_t;
typedef pthread_mutex_t *threadMutex_t;
typedef pthread_mutex_t *processLock_t;
typedef pthread_mutex_t processLockStorage_t;

typedef struct threadCondVar
{
	threadMutex_t mutex;
	pthread_cond_t *pthreadCondVar;
} *threadCondVar_t, **threadCondVarP_t;

typedef pthread_t threadHandle_t;

typedef int processHandle_t;

typedef void * (*threadFunction_t)(void *);

typedef void (*threadOnceFunction_t)(void);

typedef void *FUNCTION_RETURN_TYPE;

typedef void ONCE_FUNCTION_RETURN_TYPE;

typedef void *threadParameter_t;

typedef pthread_once_t threadOnceBlock_t;

#define THREAD_ONCE_BLOCK_INITIALISE PTHREAD_ONCE_INIT

/* if defined(_AIX) */

#elif defined(_SOLARIS)

/************************************************/
/************************************************/
/*                                              */
/*        Solaris threads implementation        */
/*                                              */
/************************************************/
/************************************************/

/*
 * typedefs 
 */
typedef thread_key_t threadKey_t;
typedef mutex_t *threadMutex_t;
typedef mutex_t *processLock_t;
typedef mutex_t processLockStorage_t;

typedef struct threadCondVar
{
	threadMutex_t mutex;
	cond_t *pthreadCondVar;
} *threadCondVar_t, **threadCondVarP_t;

typedef thread_t threadHandle_t;

typedef int processHandle_t;

typedef void * (*threadFunction_t)(void *);

typedef void (*threadOnceFunction_t)(void);

typedef void *FUNCTION_RETURN_TYPE;

typedef void ONCE_FUNCTION_RETURN_TYPE;

typedef void *threadParameter_t;

typedef struct threadOnceBlock
{
	bool_t initialised;
	mutex_t mutex;
} threadOnceBlock_t;

#define THREAD_ONCE_BLOCK_INITIALISE {FALSE, DEFAULTMUTEX}

#elif defined(_HPUX)

/************************************************/
/************************************************/
/*                                              */
/*         HPUX threads implementation          */
/*                                              */
/************************************************/
/************************************************/

/*
 * typedefs 
 */
typedef pthread_key_t threadKey_t;
typedef pthread_mutex_t *threadMutex_t;
typedef pthread_mutex_t *processLock_t;
typedef pthread_mutex_t processLockStorage_t;

typedef struct threadCondVar
{
	threadMutex_t mutex;
	pthread_cond_t *pthreadCondVar;
} *threadCondVar_t, **threadCondVarP_t;

typedef pthread_t threadHandle_t;

typedef int processHandle_t;

typedef void * (*threadFunction_t)(void *);

typedef void (*threadOnceFunction_t)(void);

typedef void *FUNCTION_RETURN_TYPE;

typedef void ONCE_FUNCTION_RETURN_TYPE;

typedef void *threadParameter_t;

typedef pthread_once_t threadOnceBlock_t;

#define THREAD_ONCE_BLOCK_INITIALISE PTHREAD_ONCE_INIT

#endif /* if defined(_HPUX) */


/*
 * function prototypes 
 */
int threadGetSlotKey(OUT threadKey_t *keyP);

int threadGetSpecificStorage(IN threadKey_t key,
							 OUT void **valuePP);

int threadSetSpecificStorage(IN threadKey_t key,
							 IN void *valueP);

int threadCreateMutex(IN bool_t initialStateLocked,
					  INOUT threadMutex_t *lockP);

int threadMutexLock(IN threadMutex_t lock);

int threadMutexUnlock(IN threadMutex_t lock);

#if defined(WIN32)
int processCreateLock(IN bool_t initialStateLocked,
					  IN char *lockName,
					  INOUT processLock_t *lockP,
					  OUT bool_t *ownerP);
#else
int processCreateLock(IN bool_t initialStateLocked,
					  IN void *sharedMemoryAddress,
					  OUT processLock_t *lockP);
#endif
int processDestroyLock(IN processLock_t *lockP);

int processLockLock(IN processLock_t lock);

int processLockUnlock(IN processLock_t lock);

int threadCreateCondVar(IN threadMutex_t mutex,
					    OUT threadCondVar_t *condvarP);

int threadCondTimedWait(IN threadCondVar_t condVar,
						IN int milliseconds);

int threadCondWait(IN threadCondVar_t condVar);

int threadCondSignal(IN threadCondVar_t condVar);

int threadCondBroadcast(IN threadCondVar_t condVar);

int threadCreateThread(IN threadFunction_t function,
					   IN threadParameter_t parameter,
					   OUT threadHandle_t *handleP);

int processCreateProcess(OUT processHandle_t *handleP,
						 IN char *parmList, 
						 ...);

int threadJoin(IN threadHandle_t handle);

int processJoin(IN processHandle_t handle);

int threadOnceBlock(IN threadOnceBlock_t *thisBlockP,
					IN threadOnceFunction_t function);

int threadGetThreadId(IN threadHandle_t handle);

int threadGetCurrentThreadId(IN void);

int threadYield(IN void);

#else  /* __THREADED */

#if defined(THREADS_DECLARE_DATA)
    void *processDataP = NULL;
#else
    extern void *processDataP;
#endif
#if 0


WINAPI and DWORD are already defined by windef.h

#if !defined(WINAPI)
    #define WINAPI
#endif
#if !defined(DWORD)
    #define DWORD
#endif
#endif

#define threadKey_t int

/*
 * typedefs 
 */
typedef int threadMutex_t;
typedef int *threadCondVar_t, **threadCondVarP_t;
typedef int *threadHandle_t, **threadHandleP_t;
#if 0
typedef int (*LPTHREAD_START_ROUTINE)(void *lpThreadParameter);
#endif
typedef void (*threadOnceFunction_t)(void);
typedef LPTHREAD_START_ROUTINE threadFunction_t;
typedef void *threadParameter_t;
typedef int threadOnceBlock_t;
#define THREAD_ONCE_BLOCK_INITIALISE 0

/*
 * no threading required so just #define most functions to NULL 
 */
#if 0
int threadGetSlotKey(OUT threadKey_t *keyP);

int threadGetSpecificStorage(IN threadKey_t key,
							 OUT void **valuePP);

int threadSetSpecificStorage(IN threadKey_t key,
							 IN void *valueP);

int threadCreateMutex(IN bool_t initialStateLocked,
					  INOUT threadMutex_t *lockP);

int threadMutexLock(IN threadMutex_t lock);

int threadMutexUnlock(IN threadMutex_t lock);

int threadCreateCondVar(IN threadMutex_t mutex,
					    OUT threadCondVar_t *condvarP);

int threadCondTimedWait(IN threadCondVar_t condVar,
						IN int milliseconds);

int threadCondWait(IN threadCondVar_t condVar);

int threadCondSignal(IN threadCondVar_t condVar);

int threadCreateThread(IN threadFunction_t function,
					   IN threadParameter_t parameter,
					   OUT threadHandle_t *handleP);

int threadJoin(IN threadHandle_t handle);

int threadOnceBlock(IN threadOnceBlock_t *thisBlockP,
					IN threadOnceFunction_t function);

int threadGetThreadId(IN threadHandle_t handle);

int threadGetCurrentThreadId(IN void);

#else

#define threadGetSlotKey(__a) 0

int threadGetSpecificStorage(IN threadKey_t key,
							 OUT void **valuePP);

int threadSetSpecificStorage(IN threadKey_t key,
							 IN void *valueP);

#define threadCreateMutex(__a, __b)
#define threadMutexLock(__a)
#define threadMutexUnlock(__a)
#define threadCreateCondVar(__a, __b)
#define threadCondTimedWait(__a, __b)
#define threadCondWait(__a)
#define threadCondSignal(__a)
#define threadCreateThread(__a, __b, __c)
#define threadJoin(__a)

int threadOnceBlock(IN threadOnceBlock_t *thisBlockP,
					IN threadOnceFunction_t function);

#define threadGetThreadId(__a) (0)
#define threadGetCurrentThreadId() (0)

#endif


#endif  /* not defined(__THREADED) */


#endif
